



//-----------------------------------------------------------------------------
// File:     MipsiLegMain.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  BioE215 MIPSI Project - Simulation of a Human Leg
// Author:   Daniel Jacobs - 5/21/07
// Last Update: 5/25/07
//-----------------------------------------------------------------------------
#include "StandardCppHeadersAndNamespace.h"
#include "ScaledBodySegmentMassProperties.h"
#include "UserForceFootVertical.h"
#include "UserForceTorsionalSpring.h"
#include "SimTKsimbody.h"
using namespace SimTK;
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
bool  DoRequiredTasks( void );
bool  WriteStringToFile(   const char outputString[], FILE *fptr )  { return fputs( outputString, fptr ) != EOF; }
bool  WriteStringToScreen( const char outputString[] )              { return WriteStringToFile( outputString, stdout ); }
bool  WriteStringToScreenAndFile( const char s[], FILE *fptr )      { return WriteStringToScreen(s)  &&  WriteStringToFile(s,fptr); }
bool  WriteDoubleToFile( double x, int precision, FILE *fptr );
bool  WriteDoubleToScreenAndFile( double x, int precision, FILE *fptr )  { return WriteDoubleToFile(x,precision,stdout) && WriteDoubleToFile(x,precision,fptr); } 
double ConvertFromRadiansToDegrees ( double angleInRadians ) { return angleInRadians * 180/3.1415926535897932384626433832795028841971; }
double ConvertFromDegreesToRadians ( double angleInDegrees ) { return angleInDegrees * 3.1415926535897932384626433832795028841971/180; }

//-----------------------------------------------------------------------------
FILE*  FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute );

MassProperties CalculateScaledBodySegmentMassProperties( bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM );
bool CalculateFootSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform  );
bool CalculateShankSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform  );
bool CalculateThighSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform  );
bool CalculateTrunkSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &insboardJointTransform );

//-----------------------------------------------------------------------------
// The executable program starts here
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{
	// Default value is program failed
	bool programSucceeded = false;

	// It is a good programming practice to do little in the main function of a program.
	// The try-catch code in this main routine catches exceptions thrown by functions in the
	// try block, e.g., catching an exception that occurs when a NULL pointer is de-referenced.
	try
	{
		// Do the required programming tasks
		programSucceeded = DoRequiredTasks();
	}
	// This catch statement handles certain types of exceptions
	catch( const exception &e )
	{
		WriteStringToScreen( "\n\n Error: Programming error encountered.\n The exception thrown is: " );
		WriteStringToScreen( e.what() );
	}
	// The exception-declaration statement (...) handles any type of exception,
	// including C exceptions and system/application generated exceptions.
	// This includes exceptions such as memory protection and floating-point violations.
	// An ellipsis catch handler must be the last handler for its try block.
	catch( ... )
	{
		WriteStringToScreen( "\n\n Error: Programming error encountered.\n An unhandled exception was thrown." );
	}

	// Give the user a chance to see a final message before exiting the program.
	WriteStringToScreen( programSucceeded ? "\n\n Program succeeded.  Press  Enter  to finish: " : "\n\n Program failed.  Press  Enter  to finish: " );
	getchar();  // Keeps the screen displayed until the user presses the Enter (or Return) key.

	// The value returned by the main function is the exit status of the program.
	// A normal program exit returns 0 (other return values usually signal an error).
	return programSucceeded == true ? 0 : 1;
}


//-----------------------------------------------------------------------------
bool  DoRequiredTasks( void )
{

	// Declare a multibody system (contains one or more force and matter sub-systems)
	MultibodySystem  mbs;

	// 0. The ground's right-handed, orthogonal x,y,z unit vectors are directed with x horizontally right and y vertically upward.
	// 1. Create a gravity vector that is straight down (in the ground's frame)
	// 2. Create a uniform gravity sub-system
	// 3. Add the gravity sub-system to the multibody system
	Vec3 gravityVector( 0, 0, 0 );
	UniformGravitySubsystem gravity( gravityVector );
	mbs.addForceSubsystem( gravity );

	// Create a matter sub-system
	SimbodyMatterSubsystem bodySubsystem;

	// My own Parameters
	const Real subjectTotalMassInKG = 58.4;
	const Real subjectTotalHeightInMM = 1.3928;
	const bool subjectIsFemale = false;


	// Array of segment Names and segment lengths
	// Note: A value of zero for a segment length causes the program to estimate the segment length using the
	//       standardBodySegmentLengthInMM * subjectTotalHeightInMM / standardBodyTotalHeightInMM.
	const char *segmentName[] =      {"Trunk",  "UpperArm",  "ForeArm",  "Thigh",   "Shank",  "Foot" };
	const Real segmentLengthInMM[] = {    0.0,     0.0,         0.0,        0.0,      0.0,      0.0 };

	// Segment 0 - Ground
	// The Ground Frame to the Hip Joint is an Identity Transform
	Transform groundToGroundTransform;

	// Segment 1 - Trunk
	MassProperties trunkMassProperties;
	Transform trunkSegmentToHipJointTransform;
	Transform trunkSegmentToGroundTransform;
	Mobilizer trunkToGroundMobilizer = Mobilizer::Planar();
	CalculateTrunkSystemTransforms( subjectIsFemale, segmentName[0], subjectTotalMassInKG, subjectTotalHeightInMM, segmentLengthInMM[0], trunkMassProperties, trunkSegmentToGroundTransform, trunkSegmentToHipJointTransform	);

	// Segment 2 - Thigh
	MassProperties thighMassProperties;
	Transform thighSegmentToHipJointTransform;
	Transform thighSegmentToKneeJointTransform;
	Mobilizer thighToTrunkMobilizer = Mobilizer::Pin();
	CalculateThighSystemTransforms( subjectIsFemale, segmentName[3], subjectTotalMassInKG, subjectTotalHeightInMM, segmentLengthInMM[3], thighMassProperties, thighSegmentToHipJointTransform, thighSegmentToKneeJointTransform );

	// Segment 3 - Shank
	MassProperties shankMassProperties;
	Transform shankSegmentToKneeJointTransform;
	Transform shankSegmentToAnkleJointTransform;
	Mobilizer shankToThighMobilizer = Mobilizer::Pin();
	CalculateShankSystemTransforms( subjectIsFemale, segmentName[4], subjectTotalMassInKG, subjectTotalHeightInMM, segmentLengthInMM[4], shankMassProperties, shankSegmentToKneeJointTransform, shankSegmentToAnkleJointTransform );

	// Segment 4 - Foot
	MassProperties footMassProperties;
	Transform footSegmentToAnkleJointTransform;
	Transform footSegmentToNothingTransform;
	Mobilizer footToShankMobilizer = Mobilizer::Pin();
	CalculateFootSystemTransforms( subjectIsFemale, segmentName[5], subjectTotalMassInKG, subjectTotalHeightInMM, segmentLengthInMM[5], footMassProperties, footSegmentToAnkleJointTransform, footSegmentToNothingTransform );

	// Add the Segments to the Matter SubSystem
	//const BodyId bookBodyId = book.addRigidBody( bookMassProperties, inboardFrameTransformFromBook, GroundId, outboardFrameTransformFromGround, bookToGroundMobilizer );
	const BodyId trunkBodyId = bodySubsystem.addRigidBody( trunkMassProperties, trunkSegmentToGroundTransform, GroundId, groundToGroundTransform, trunkToGroundMobilizer ); 
	const BodyId thighBodyId = bodySubsystem.addRigidBody( thighMassProperties, thighSegmentToHipJointTransform,  trunkBodyId,  trunkSegmentToHipJointTransform, thighToTrunkMobilizer ); 
	const BodyId shankBodyId = bodySubsystem.addRigidBody( shankMassProperties, shankSegmentToKneeJointTransform, thighBodyId, thighSegmentToKneeJointTransform, shankToThighMobilizer ); 
	const BodyId  footBodyId = bodySubsystem.addRigidBody(  footMassProperties, footSegmentToAnkleJointTransform, shankBodyId,  shankSegmentToAnkleJointTransform, footToShankMobilizer ); 

	// Add the matter (body) sub-system to the system.
	mbs.setMatterSubsystem( bodySubsystem );


/*
	Real initialTrunkAngleInRads = ConvertFromDegreesToRadians( 85 );
	Real initialThighAngleInRads = ConvertFromDegreesToRadians( -120 );
	Real initialShankAngleInRads = ConvertFromDegreesToRadians( -60 );
	Real initialFootAngleInRads = ConvertFromDegreesToRadians( 40 );

*/

	Real initialTrunkAngleInRads = ConvertFromDegreesToRadians( 65 );
	Real initialThighAngleInRads = ConvertFromDegreesToRadians( -100 );
	Real initialShankAngleInRads = ConvertFromDegreesToRadians( -85 );
	Real initialFootAngleInRads = ConvertFromDegreesToRadians( 120 );

	// Add an air-resistance force sub-system to this multi-body system.
	GeneralForceElements userForceElements;
	mbs.addForceSubsystem( userForceElements );



	// Although "new" was used to allocate this UserForce, do not "delete" it.
	// This bug will be fixed in the next version of Simbody so it can take an object from the stack or heap.
	// For now, addUserForce takes ownership of the allocated item and takes care of deleting it at the end.
	// UserForceCostHorizontal *sinusoidalForceOnApple = new UserForceCostHorizontal( appleBodyId );
	// userForceElements.addUserForce( sinusoidalForceOnApple );
	
	Real springConstant = 100;
	Real dampingRatio = .4;
	Real groundHeight = -1;
/*	Vec3 rHeel(-.050,-.02,0);
	Vec3 rToe(.050,-.02,0);
	UserForceFootVertical *footForce = new UserForceFootVertical(footBodyId, GroundId, springConstant, dampingRatio, rHeel, rToe, groundHeight); 
	userForceElements.addUserForce( footForce );

	springConstant = 10000;
	UserForceTorsionalSpring *hipTorque = new UserForceTorsionalSpring(trunkBodyId, thighBodyId, springConstant, initialTrunkAngleInRads-initialThighAngleInRads);
	UserForceTorsionalSpring *kneeTorque = new UserForceTorsionalSpring(thighBodyId, shankBodyId, springConstant, initialThighAngleInRads-initialShankAngleInRads);
	UserForceTorsionalSpring *ankleTorque = new UserForceTorsionalSpring(shankBodyId, footBodyId, springConstant, initialShankAngleInRads-initialFootAngleInRads);

//	UserForceTorsionalSpring *hipTorque = new UserForceTorsionalSpring(trunkBodyId, thighBodyId, springConstant, 0);
//	UserForceTorsionalSpring *kneeTorque = new UserForceTorsionalSpring(thighBodyId, shankBodyId, springConstant, 0);
//	UserForceTorsionalSpring *ankleTorque = new UserForceTorsionalSpring(shankBodyId, footBodyId, springConstant, 0);
	userForceElements.addUserForce(hipTorque);
	userForceElements.addUserForce(kneeTorque);
	userForceElements.addUserForce(ankleTorque);
*/
	
//	bodySubsystem.addWeldConstraint(GroundId, groundToGroundTransform, trunkBodyId, groundToGroundTransform);


	// Create a state for this system.
	// Define appropriate states for this multi-body system.
	// Set the initial time to 0.0
	State s;
	mbs.realize( s );
	s.setTime( 0.0 );

	// Set Mobilizers
	// Set the Trunk Planar Mobilizer

	///The generalized
	/// coordinates are rotation about the shared z axis of the Mb and M
	/// frame, translation along the Mb frame's x axis, and translation along
	/// its y axis, in that order.
	bodySubsystem.setMobilizerQ( s, trunkBodyId, 0,  initialTrunkAngleInRads );
	bodySubsystem.setMobilizerQ( s, trunkBodyId, 1,  0.0 );
	bodySubsystem.setMobilizerQ( s, trunkBodyId, 2,  0.0 );

	bodySubsystem.setMobilizerU( s, trunkBodyId, 0,  0.0 );
	bodySubsystem.setMobilizerU( s, trunkBodyId, 1,  0.0 );
	bodySubsystem.setMobilizerU( s, trunkBodyId, 2,  0.0 );

	// Set the Thigh Pin Mobilizer
	/// One mobility -- rotation about the common z axis of the inboard
	/// and outboard mobilizer frames.
	bodySubsystem.setMobilizerQ( s, thighBodyId, 0,  initialThighAngleInRads );
	bodySubsystem.setMobilizerU( s, thighBodyId, 0,  0.0 );

	// Set the Shank Pin Mobilizer
	bodySubsystem.setMobilizerQ( s, shankBodyId, 0,  initialShankAngleInRads );
	bodySubsystem.setMobilizerU( s, shankBodyId, 0,  0.0 );

	// Set the Foot Pin Mobilizer
	bodySubsystem.setMobilizerQ( s, footBodyId, 0,  initialFootAngleInRads );
	bodySubsystem.setMobilizerU( s, footBodyId, 0,  0.0 );

	// Create a study using the Runge Kutta Merson integrator (alternately use the CPodesIntegrator)
	RungeKuttaMerson myStudy( mbs, s );

	// Set the numerical accuracy for the integrator
	myStudy.setAccuracy( 1.0E-7 );

	// The next statement does lots of accounting
	myStudy.initialize();

	// Open a file to record the simulation results (they are also displayed on screen)
	// Open a file for writing (return false if cannot open file)
	const char *outputFileName = "BioEPlanarLeg.txt";
	FILE *outputFile = FileOpenWithMessageIfCannotOpen( outputFileName, "w" );
	if( !outputFile ) return false;
	WriteStringToScreenAndFile(   "time    yTrunk   yThigh    yShank  yFoot rHeelInFoot  rHeelInGround\n",  outputFile );

	// Visualize results with VTKReporter
	VTKReporter animationResults( mbs );

	// For visualization purposes only, create 4 cylinders
	DecorativeCylinder trunkVisual = DecorativeCylinder(.05, .21276);
	trunkVisual.setColor(Red);     // Can also specify a Vec3 with rgb which scale from 0 to 1
	trunkVisual.setOpacity(0.4);   // 0.0 is solid and 1.0 is transparent

	DecorativeCylinder thighVisual = DecorativeCylinder(.040, .16888);
	thighVisual.setColor(Blue);     // Can also specify a Vec3 with rgb which scale from 0 to 1
	thighVisual.setOpacity(0.4);   // 0.0 is solid and 1.0 is transparent

	DecorativeCylinder shankVisual = DecorativeCylinder(.040, .1736);
	shankVisual.setColor(Green);     // Can also specify a Vec3 with rgb which scale from 0 to 1
	shankVisual.setOpacity(0.4);   // 0.0 is solid and 1.0 is transparent

	DecorativeCylinder footVisual = DecorativeCylinder(.025, .10324);
	footVisual.setColor(Gray);     // Can also specify a Vec3 with rgb which scale from 0 to 1
	footVisual.setOpacity(0.4);   // 0.0 is solid and 1.0 is transparent

	// Decorate the apple with the red sphere at the apple's origin
	const Rotation  identityOrientation; //( 1,0,0, 0,1,0, 0,0,1 );
	const Rotation  zRotationBy90Degrees = identityOrientation.aboutZ(ConvertFromDegreesToRadians(-90));
	const Vec3      nullLocation( 0, 0, 0 );

	const Transform identityTransform( identityOrientation, nullLocation );
	const Transform rotationBy90DegreesZTransform( zRotationBy90Degrees, nullLocation );

	animationResults.addDecoration( trunkBodyId, rotationBy90DegreesZTransform, trunkVisual );
	animationResults.addDecoration( thighBodyId, rotationBy90DegreesZTransform, thighVisual );
	animationResults.addDecoration( shankBodyId, rotationBy90DegreesZTransform, shankVisual );
	animationResults.addDecoration(  footBodyId, rotationBy90DegreesZTransform,  footVisual );


	//Ground
	Vec3 groundDimensions(1,.002,1);
	DecorativeBrick groundBrick = DecorativeBrick(groundDimensions);
	groundBrick.setColor(Blue);
	groundBrick.setOpacity(0.5);
	const Vec3 groundLocation(0,groundHeight,0);
	const Transform groundTransform(identityOrientation, groundLocation);
	animationResults.addDecoration(GroundId, groundTransform, groundBrick);


	// Set the numerical integration step and the time for the simulation to run
	const Real dt = 0.002;
	const Real finalTime = 4.0;

	// Run the simulation and print the results
	while( 1 )
	{
		// Query for results to be printed
		Real time = s.getTime();

		// Extract the movement of the center of mass of each object;
		// Extract the apple's y-location from this vector.
//		const Vec3 rHeelInGroundFrame = bodySubsystem.calcBodyPointLocationInBody(s, footBodyId, rHeel, GroundId);
		const Vec3 trunkLocationVector = bodySubsystem.calcBodyOriginLocationInBody( s, trunkBodyId, GroundId );
		const Vec3 thighLocationVector = bodySubsystem.calcBodyOriginLocationInBody( s, thighBodyId, GroundId );
		const Vec3 shankLocationVector = bodySubsystem.calcBodyOriginLocationInBody( s, shankBodyId, GroundId );
		const Vec3  footLocationVector = bodySubsystem.calcBodyOriginLocationInBody( s,  footBodyId, GroundId );
		Real trunkLocationY = trunkLocationVector[1];
		Real thighLocationY = thighLocationVector[1];
		Real shankLocationY = shankLocationVector[1];
		Real  footLocationY =  footLocationVector[1];
		Vector angles = bodySubsystem.getQ(s);
		Real trunkAngle = bodySubsystem.getMobilizerQ(s, trunkBodyId, 0);
		Real footAngle = bodySubsystem.getMobilizerQ(s,footBodyId,0);
		Real angleA = bodySubsystem.getMobilizerQ(s,trunkBodyId,0);
		Real angleB = bodySubsystem.getMobilizerQ(s,thighBodyId,0);
		Real currentAngle = angleA - angleB;
		Real torqueH = -springConstant*(currentAngle);


		// Print results to file
		WriteDoubleToScreenAndFile( time,				2, outputFile );
		WriteDoubleToScreenAndFile( trunkLocationY,	4, outputFile );
		WriteDoubleToScreenAndFile( thighLocationY,	4, outputFile );
		WriteDoubleToScreenAndFile( shankLocationY,	4, outputFile );
		WriteDoubleToScreenAndFile(  footLocationY,	4, outputFile );
//		WriteDoubleToScreenAndFile( rHeel[0], 4, outputFile);
//		WriteDoubleToScreenAndFile( rHeel[1], 4, outputFile);
//		WriteDoubleToScreenAndFile( rHeel[2], 4, outputFile);
//		WriteDoubleToScreenAndFile( rHeelInGroundFrame[0], 4, outputFile);
//		WriteDoubleToScreenAndFile( rHeelInGroundFrame[1], 4, outputFile);
//		WriteDoubleToScreenAndFile( rHeelInGroundFrame[2], 4, outputFile);
		WriteDoubleToScreenAndFile( angles[0],4,outputFile);
		WriteDoubleToScreenAndFile( trunkAngle, 4, outputFile);
		WriteDoubleToScreenAndFile( angles[5],4,outputFile);
		WriteDoubleToScreenAndFile( footAngle, 4, outputFile);
		WriteDoubleToScreenAndFile(angleA, 4, outputFile);
		WriteDoubleToScreenAndFile(angleB, 4, outputFile);
		WriteDoubleToScreenAndFile(torqueH, 4, outputFile);

		


		WriteStringToScreenAndFile( "\n",                outputFile );

		// Animate the results for this step
		animationResults.report(s);

		// Check if integration has completed
		if( time >= finalTime ) break;

		// Increment time step
		myStudy.step( time + dt);
	}


	// Calculations performed successfully - close output file
	WriteStringToScreen( "\n\n Calculations were successfuly performed.\n Results are in the file: " );
	WriteStringToScreen( outputFileName );
	fclose( outputFile );
	outputFile = NULL;

	return true;
}



//-----------------------------------------------------------------------------
MassProperties CalculateScaledBodySegmentMassProperties( bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM )
{
	// Get the standard body segment mass properties
	const StandardBodySegmentMassProperties*  standardSegment = StandardBodySegmentMassProperties::getStandardHumanBodySegmentMassProperties(subjectIsFemale, segmentName);
	// if( !standardSegment ) return false;

	// Scale the properties with the given information
	const ScaledBodySegmentMassProperties scaledSegment( *standardSegment, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM );

	// Write the segment length to the file
	Real segmentLength = scaledSegment.getSegmentLengthInMM();

	// Write the segment mass to the file
	Real segmentMass = scaledSegment.getMassInKG();

	// Get the segment's distance from its origin anatomical marker to its mass center distance
	Real segmentMassCenterLength = scaledSegment.getMassCenterLengthInMM();

	// Get       the segment x-radius of gyration about its mass center to the file
	// Calculate the segment x-moment of inertia  about its mass center to the file
	Real kxx = scaledSegment.getXRadiusOfGyrationInMM();
	Real Ixx = segmentMass*kxx*kxx;

	// Get       the segment y-radius of gyration about its mass center to the file
	// Calculate the segment y-moment of inertia  about its mass center to the file
	Real kyy = scaledSegment.getYRadiusOfGyrationInMM();
	Real Iyy = segmentMass*kyy*kyy;

	// Get       the segment z-radius of gyration about its mass center to the file
	// Calculate the segment z-moment of inertia  about its mass center to the file
	Real kzz = scaledSegment.getZRadiusOfGyrationInMM();
	Real Izz = segmentMass*kzz*kzz;

	// The location of the segment's center of mass is a vector from the segments's
	// origin expressed in the "x, y, z"' unit vectors fixed in the segment's frame.
	// Example: The vector(0,0,0) locates the segment's center of mass at the segment's origin.
	const Vec3  segmentCenterOfMassLocation( 0, 0, 0 );

	// Create the segment's inertia matrix about its origin for the "x, y, z" unit vectors fixed in the apple's frame.
	// Note: The 3x3 inertia matrix is symmetric - so only 6 elements need to be defined.
	// Ixx, Iyy, Izz  are moments  of inertia (    diagonal terms in the matrix)
	// Ixy, Ixz, Iyz  are products of inertia (off-diagonal terms in the matrix)
	const Real  Ixy = 0,  Ixz = 0,   Iyz = 0;
	const Inertia  segmentInertiaMatrix( Ixx, Iyy, Izz, Ixy, Ixz, Iyz );

	segmentMass /= 1000;
	segmentMassCenterLength /= 1000;
	kxx /= 1000;
	kyy /= 1000;
	kzz /= 1000;
	Ixx /= 1000000;
	Iyy /= 1000000;
	Izz /= 1000000;

	// The MassProperties class holds the mass, center of mass, and inertia properties of a rigid body.
	// Although the next line creates an instance of the MassProperties class for the apple,
	// it does not get associated with the apple until the  addRigidBody  method.
	MassProperties segmentMassProperties( segmentMass, segmentCenterOfMassLocation, segmentInertiaMatrix );

	return segmentMassProperties;
}

//-----------------------------------------------------------------------------
bool CalculateTrunkSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform )
{

	// Get the standard body segment mass properties
	const StandardBodySegmentMassProperties*  standardSegment = standardSegment->getStandardHumanBodySegmentMassProperties(subjectIsFemale, segmentName);
	// if( !standardSegment ) return false;

	// Scale the properties with the given information
	const ScaledBodySegmentMassProperties scaledSegment( *standardSegment, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM );

	// Write the segment length to the file
	Real segmentLength = scaledSegment.getSegmentLengthInMM()/1000;

	// Write the segment mass to the file
	Real segmentMass = scaledSegment.getMassInKG();

	// Get the segment's distance from its origin anatomical marker to its mass center distance
	Real segmentMassCenterLength = scaledSegment.getMassCenterLengthInMM()/1000;

	// Update Passed In Address Values

	segmentMassProperties = CalculateScaledBodySegmentMassProperties( subjectIsFemale, segmentName, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM);

	// Build Transformation from Trunk Segment
	// Trunk is Inboard Transform for Hip Joint From Thigh
	// Trunk is Outboard for Ground Transform which is Identity

	// Transformation from Trunk origin to Ground in Trunk Frame
	const Rotation    inFrameOrientationInTrunk;
	const Vec3        inFrameOriginLocationFromTrunkOrigin(-(segmentLength-segmentMassCenterLength),0,0);
	const Transform   inFrameTransformFromTrunk( inFrameOrientationInTrunk, inFrameOriginLocationFromTrunkOrigin );

	inboardJointTransform = inFrameTransformFromTrunk;

	// Transformation from Trunk Origin to Hip Joint in Trunk Frame
	const Rotation    outFrameOrientationInTrunk;
	const Vec3        outFrameOriginLocationFromTrunkOrigin(0,0,0);
	const Transform   outFrameTransformFromTrunk( outFrameOrientationInTrunk, outFrameOriginLocationFromTrunkOrigin );

	outboardJointTransform = outFrameTransformFromTrunk;

	return true;
}

//-----------------------------------------------------------------------------
bool CalculateThighSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform )
{

	// Get the standard body segment mass properties
	const StandardBodySegmentMassProperties*  standardSegment = standardSegment->getStandardHumanBodySegmentMassProperties(subjectIsFemale, segmentName);
	// if( !standardSegment ) return false;

	// Scale the properties with the given information
	const ScaledBodySegmentMassProperties scaledSegment( *standardSegment, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM );

	// Write the segment length to the file
	Real segmentLength = scaledSegment.getSegmentLengthInMM()/1000;

	// Write the segment mass to the file
	Real segmentMass = scaledSegment.getMassInKG();

	// Get the segment's distance from its origin anatomical marker to its mass center distance
	Real segmentMassCenterLength = scaledSegment.getMassCenterLengthInMM()/1000;

	// Update Passed In Address Values

	segmentMassProperties = CalculateScaledBodySegmentMassProperties( subjectIsFemale, segmentName, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM);

	// Build Transformation for Thigh Segment
	// Thigh is Inboard Transform for Knee Joint From Shank
	// Thigh is Outboard Transform for Hip Joint from Trunk

	// Transformation from Thigh Origin Frame to Knee Joint in Thigh Frame
	const Rotation    inFrameOrientationInThigh;
	const Vec3        inFrameOriginLocationFromThighOrigin(segmentLength-segmentMassCenterLength,0,0);
	const Transform   inFrameTransformFromThigh( inFrameOrientationInThigh, inFrameOriginLocationFromThighOrigin );

	inboardJointTransform = inFrameTransformFromThigh;

	// Transformation from Thigh Origin to Hip Joint in Thigh Frame
	const Rotation    outFrameOrientationInThigh;
	const Vec3        outFrameOriginLocationFromThighOrigin(-segmentMassCenterLength,0,0);
	const Transform   outFrameTransformFromThigh( outFrameOrientationInThigh, outFrameOriginLocationFromThighOrigin );

	outboardJointTransform = outFrameTransformFromThigh;

	return true;

}

//-----------------------------------------------------------------------------
bool CalculateShankSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform  )
{
	// Get the standard body segment mass properties
	const StandardBodySegmentMassProperties*  standardSegment = standardSegment->getStandardHumanBodySegmentMassProperties(subjectIsFemale, segmentName);
	// if( !standardSegment ) return false;

	// Scale the properties with the given information
	const ScaledBodySegmentMassProperties scaledSegment( *standardSegment, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM );

	// Write the segment length to the file
	Real segmentLength = scaledSegment.getSegmentLengthInMM()/1000;

	// Write the segment mass to the file
	Real segmentMass = scaledSegment.getMassInKG();

	// Get the segment's distance from its origin anatomical marker to its mass center distance
	Real segmentMassCenterLength = scaledSegment.getMassCenterLengthInMM()/1000;

	// Update Passed In Address Values

	segmentMassProperties = CalculateScaledBodySegmentMassProperties( subjectIsFemale, segmentName, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM);

	// Build Transformation for Shank Segment
	// Shank is Inboard Transform for Ankle Joint From Foot
	// Shank is Outboard Transform for Knee Joint from Thigh

	// Transformation from Shank Origin to Ankle Joint in Shank Frame
	const Rotation    inFrameOrientationInShank;
	const Vec3        inFrameOriginLocationFromShankOrigin(segmentLength-segmentMassCenterLength,0,0);
	const Transform   inFrameTransformFromShank( inFrameOrientationInShank, inFrameOriginLocationFromShankOrigin );

	inboardJointTransform = inFrameTransformFromShank;

	// Transformation from Shank Origin to Knee Joint in Shank Frame
	const Rotation    outFrameOrientationInShank;
	const Vec3        outFrameOriginLocationFromShankOrigin(-segmentMassCenterLength,0,0);
	const Transform   outFrameTransformFromShank( outFrameOrientationInShank, outFrameOriginLocationFromShankOrigin );

	outboardJointTransform = outFrameTransformFromShank;

	return true;
}

//-----------------------------------------------------------------------------
bool CalculateFootSystemTransforms(bool subjectIsFemale, const char *segmentName, Real subjectTotalMassInKG, Real subjectTotalHeightInMM, Real subjectSegmentLengthInMM, MassProperties &segmentMassProperties, Transform &outboardJointTransform, Transform &inboardJointTransform  )
{

	// Get the standard body segment mass properties
	const StandardBodySegmentMassProperties*  standardSegment = standardSegment->getStandardHumanBodySegmentMassProperties(subjectIsFemale, segmentName);
	// if( !standardSegment ) return false;

	// Scale the properties with the given information
	const ScaledBodySegmentMassProperties scaledSegment( *standardSegment, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM );

	// Write the segment length to the file
	Real segmentLength = scaledSegment.getSegmentLengthInMM()/1000;

	// Write the segment mass to the file
	Real segmentMass = scaledSegment.getMassInKG();

	// Get the segment's distance from its origin anatomical marker to its mass center distance
	Real segmentMassCenterLength = scaledSegment.getMassCenterLengthInMM()/1000;

	// Update Passed In Address Values

	segmentMassProperties = CalculateScaledBodySegmentMassProperties( subjectIsFemale, segmentName, subjectTotalMassInKG, subjectTotalHeightInMM, subjectSegmentLengthInMM);

	// Build Transformation for Foot Segment
	// Foot is Inboard Transform for Nothing
	// Foot is Outboard Transform for Ankle Joint from Shank

	// Transformation from Foot Origin to Nothing (Identity)
	const Rotation    inFrameOrientationInFoot;
	const Vec3        inFrameOriginLocationFromFootOrigin(0,0,0);
	const Transform   inFrameTransformFromFoot( inFrameOrientationInFoot, inFrameOriginLocationFromFootOrigin );

	inboardJointTransform = inFrameTransformFromFoot;

	// Transformation from Thigh Coordinate Frame to Hip Joint in Thigh Frame
	const Rotation    outFrameOrientationInFoot;
	const Vec3        outFrameOriginLocationFromFootOrigin(-segmentMassCenterLength,0,0);
	const Transform   outFrameTransformFromFoot( outFrameOrientationInFoot, outFrameOriginLocationFromFootOrigin );

	outboardJointTransform = outFrameTransformFromFoot;

	return true;

}



//-----------------------------------------------------------------------------
FILE*  FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute )
{
	// Try to open the file
	FILE *Fptr1 = fopen( filename, attribute );

	// If unable to open the file, issue a message
	if( !Fptr1 )
	{
		WriteStringToScreen( "\n\n Unable to open the file: " );
		WriteStringToScreen( filename );
		WriteStringToScreen( "\n\n" );
	}

	return Fptr1;
}
//-----------------------------------------------------------------------------
bool  WriteDoubleToFile( double x, int precision, FILE *fptr )
{
	// Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
	// Next, calculate the field width so it includes one extra space to the right of the number.
	if( precision < 0 || precision > 17 ) precision = 5;
	int fieldWidth = precision + 8;

	// Create the format specifier and print the number
	char format[20];
	sprintf( format, " %%- %d.%dE", fieldWidth, precision );
	return fprintf( fptr, format, x ) >= 0;
}


