//-----------------------------------------------------------------------------
// File:     UserForceFootVertical.h
// Class:    UserForceFootVertical.h
// Parent:   GeneralForceElements
// Children: None
// Purpose:  Applies ground reaction forces on the heel and toe of a foot
// Author:   Sanjay Dastoor - 06/13/07
//-----------------------------------------------------------------------------
#ifndef __USERFORCEFOOTVERTICAL_H__
#define __USERFORCEFOOTVERTICAL_H__
//-----------------------------------------------------------------------------
#include "StandardCppHeadersAndNamespace.h"
#include "SimTKsimbody.h"
using namespace SimTK;

//--------------------------------------------------------------------------
// User-defined classes for adding forces/torques are constructed as follows:
// 1. Create a constructor with whatever arguments make sense for the force or torque and copy the arguments into class data.
// 2. Create a clone method (all clone methods are identical except for the class name appearing after "new")
// 3. Create a calc  method (the arguments and return type for all calc methods are identical).
//    The code in the calc method is specific to the calculation of force or torque.
// Note: The set of all forces is replaced by an equivalent set, consisting of a torque
//       that is equal to the moment of the forces about the body's origin together
//       with the resultant of the forces applied at the body's origin.
//--------------------------------------------------------------------------
class UserForceFootVertical : public GeneralForceElements::UserForce 
{
public:
   // Constructor is explicit
   explicit UserForceFootVertical( BodyId footId, BodyId groundId, Real springConstant, Real dampingRatio, Vec3 rHeel, Vec3 rToe, Real groundHeight)
   { 
      myFootId = footId;
	  myGroundId = groundId;
	  mySpringConstant = springConstant;
	  myDampingRatio = dampingRatio;
	  myRHeelInFootFrame = rHeel;
	  myRToeInFootFrame = rToe;
	  myGroundHeight = groundHeight;
   } 

   // The clone method is used internally by Simbody (required by virtual parent class)
   UserForce*  clone() const  { return new UserForceFootVertical(*this); }

   // The calc method is where forces or torques are calculated (required by virtual parent class)
   void  calc( const MatterSubsystem& matter,              // Input information (matter)
               const State&           state,               // Input information (current state)
               Vector_<SpatialVec>&   bodyForces,          // Forces and torques on bodies
               Vector_<Vec3>&         particleForces,      // Forces on particles (currently unused)
               Vector&                mobilityForces,      // Generalized forces 
               Real&                  pe ) const           // For forces with a potential energy
    {
        // Query the matter subsystem for the body's origin velocity in ground.
        // This vector is expressed in the ground's "x,y,z" unit vectors.
        Real forceH = 0;
		Real torqueH = 0;
		const Vec3 rHeelFromGroundInGroundFrame = matter.calcBodyPointLocationInBody(state, myFootId, myRHeelInFootFrame, myGroundId);
//		const Vec3 vHeelInGroundFrame = matter.calcBodyFixedPointVelocityInBody(state, myFootId, myRHeelInFootFrame, myGroundId);
		const Vec3 vFootOriginInGroundFrame = matter.calcBodyOriginVelocityInBody(state, myFootId, myGroundId);
		const Vec3 wFootInGroundFrame = matter.calcBodyAngularVelocityInBody(state,myFootId, myGroundId);
		const Vec3 vHeelInGroundFrame = vFootOriginInGroundFrame + cross(myRHeelInFootFrame, wFootInGroundFrame);
		const Vec3 rHeelFromFootOriginInGroundFrame = matter.calcBodyVectorInBody(state,myFootId, myRHeelInFootFrame,myGroundId);

        Real forceT = 0;
		Real torqueT = 0;
		const Vec3 rToeFromGroundInGroundFrame = matter.calcBodyPointLocationInBody(state, myFootId, myRToeInFootFrame, myGroundId);
//		const Vec3 vHeelInGroundFrame = matter.calcBodyFixedPointVelocityInBody(state, myFootId, myRHeelInFootFrame, myGroundId);
		const Vec3 vToeInGroundFrame = vFootOriginInGroundFrame + cross(myRToeInFootFrame, wFootInGroundFrame);
		const Vec3 rToeFromFootOriginInGroundFrame = matter.calcBodyVectorInBody(state,myFootId, myRToeInFootFrame,myGroundId);


		Real yPositionH = rHeelFromGroundInGroundFrame[1];
		Real yVelocityH = vHeelInGroundFrame[1];
		Real momentArmH = rHeelFromFootOriginInGroundFrame[0];

		Real yPositionT = rToeFromGroundInGroundFrame[1];
		Real yVelocityT = vToeInGroundFrame[1];
		Real momentArmT = rToeFromFootOriginInGroundFrame[0];

		//Get current time from state
		Real simulationTime = state.getTime();

		if ((yPositionH - myGroundHeight) < 0)
		{
			forceH -= mySpringConstant*(yPositionH-myGroundHeight);
			torqueH += forceH*momentArmH;
			if (yVelocityH < 0)			
				forceH -= myDampingRatio*yVelocityH;
		}

		if ((yPositionT - myGroundHeight) < 0)
		{
			forceT -= mySpringConstant*(yPositionT-myGroundHeight);
			torqueT += forceT*momentArmT;
			if (yVelocityT < 0)			
				forceT -= myDampingRatio*yVelocityT;
		}

        // Get the proper memory location to increment the force and/or torque.
        // bodiesForces is a Vec6 whose elements are two Vec3.
        // The elements of the first  Vec3 are Tx, Ty, Tz  (expressed in the ground's "x,y,z").
        // The elements of the second Vec3 are Fx, Fy, Fz  (expressed in the ground's "x,y,z").
        SpatialVec& bodiesForces = bodyForces[ myFootId ];
        Vec3& torqueSum = bodiesForces[0];
        Vec3& forceSum  = bodiesForces[1];

        // Increment the sum of all forces on this body (other force subsystems may also add forces/torque)
        // torqueSum[0] += 0;       // Increment torque in the ground's x-direction.
        // torqueSum[1] += 0;       // Increment torque in the ground's y-direction.
         torqueSum[2] += torqueH;       // Increment torque in the ground's z-direction.
		 torqueSum[2] += torqueT;
        // forceSum[0]  += 0;       // Increment force  in the ground's x-direction.
         forceSum[1]  += forceH;    // Increment force  in the ground's y-direction.
		 forceSum[1] += forceT;
        // forceSum[0]  += 0;       // Increment force  in the ground's z-direction.
    }

private:
    BodyId  myFootId;
	BodyId  myGroundId;
	Real    mySpringConstant;
	Real    myDampingRatio;
	Vec3    myRHeelInFootFrame;
	Vec3    myRToeInFootFrame;
	Real    myGroundHeight;
};
//-----------------------------------------------------------------------------
#endif /* __USERFORCEFOOTVERTICAL_H__ */
//-----------------------------------------------------------------------------
