//-----------------------------------------------------------------------------
// File:     Book.cpp
// Class:    None
// Parent:   None
// Children: None
// Purpose:  Simulates a book falling in space onto the ground
//-----------------------------------------------------------------------------
// The following are standard C/C++ header files.
// If a filename is enclosed inside < >  it means the header file is in the Include directory.
// If a filename is enclosed inside " "  it means the header file is in the current directory.
#include <ctype.h>      // Character Types
#include <math.h>       // Mathematical Constants
#include <stdarg.h>     // Variable Argument Lists
#include <stdio.h>      // Standard Input/Output Functions
#include <stdlib.h>     // Utility Functions
#include <string.h>     // String Operations
#include <signal.h>     // Signals (Contol-C + Unix System Calls)
#include <setjmp.h>     // Nonlocal Goto (For Control-C)
#include <time.h>       // Time and Date information
#include <assert.h>     // Verify Program Assertion
#include <errno.h>      // Error Codes (Used in Unix system())
#include <float.h>      // Floating Point Constants
#include <limits.h>     // Implementation Constants
#include <stddef.h>     // Standard Definitions
#include <exception>    // Exception handling (e.g., try, catch throw)
//-----------------------------------------------------------------------------
#include "SimTKsimbody.h"
#include "UserForceFootVertical.h"
using namespace SimTK;
using namespace std;
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Prototypes for local functions (functions not called by code in other files)
//-----------------------------------------------------------------------------
bool  SimulateProjectile( void );
bool  WriteStringToFile(   const char outputString[], FILE *fptr )  { return fputs( outputString, fptr ) != 0; }
bool  WriteStringToScreen( const char outputString[] )              { return WriteStringToFile( outputString, stdout ); }
bool  WriteDoubleToFile( double x, int precision, FILE *fptr );
FILE*  FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute );
double ConvertFromRadiansToDegrees ( double angleInRadians ) { return angleInRadians * 180/3.1415926535897932384626433832795028841971; }
double ConvertFromDegreesToRadians ( double angleInDegrees ) { return angleInDegrees * 3.1415926535897932384626433832795028841971/180; }


//-----------------------------------------------------------------------------
// The executable program starts here
//-----------------------------------------------------------------------------
int  main( int numberOfCommandLineArguments, char *arrayOfCommandLineArguments[] )
{
	// Simulate the multibody system
	bool simulationSucceeded = SimulateProjectile();

	// Keep the screen displayed until the user presses the Enter key
	WriteStringToScreen( "\n\n Press  Enter  to terminate the program: " );
	getchar();

	// The value returned by the main function is the exit status of the program.
	// A normal program exit returns 0 (other return values usually signal an error).
	return simulationSucceeded == true ? 0 : 1;
}


//-----------------------------------------------------------------------------
bool  SimulateProjectile( void )
{
	// Declare a multibody system (contains one or more force and matter sub-systems)
	MultibodySystem  mbs;

	// Since there is no gravity acting on the book (just spinning in space), set gravityVector to 0
	Vec3 gravityVector( 0, -9.80, 0 );
	UniformGravitySubsystem gravity( gravityVector );
	mbs.addForceSubsystem( gravity );

	// Create a matter sub-system (the book)
	SimbodyMatterSubsystem  book;

	// Create the mass, center of mass, and inertia properties for the book
	const Real  massOfBook = 1;

	// The location of the book's center of mass is a vector from the book's
	// origin expressed in the "x, y, z"' unit vectors fixed in the book's frame.
	const Vec3  bookCenterOfMassLocation( 0, 0, 0 );

	// Create the book's inertia matrix about its origin for the "x, y, z" unit vectors fixed in the book's frame.
	// Note: The 3x3 inertia matrix is symmetric - so only 6 elements need to be defined.
	// Ixx, Iyy, Izz  are moments  of inertia (    diagonal terms in the matrix)
	// Ixy, Ixz, Iyz  are products of inertia (off-diagonal terms in the matrix)
	const Real  Ixx = 3,  Iyy = 1,   Izz = 2;
	const Real  Ixy = 0,  Ixz = 0,   Iyz = 0;
	const Inertia  bookInertiaMatrix( Ixx, Iyy, Izz, Ixy, Ixz, Iyz );

	// The MassProperties class holds the mass, center of mass, and inertia properties of a rigid body.
	MassProperties  bookMassProperties( massOfBook, bookCenterOfMassLocation, bookInertiaMatrix );

	// The orientation and position of the  outboard frame  from the ground's frame is specified below.
	// The outboard frame's axes are aligned with the ground's axes and its origin is coincident with the ground's origin.
	// In other words, for this simple problem the outboard frame and the ground frame are identical.
	const Transform outboardFrameTransformFromGround;  // The default constructor is the identity transform

	// The orientation and position of the  inboard  frame  from the book's  frame is specified below.
	// The inboard frame's  axes are aligned with the book's  axes and its origin is coincident with the book's  origin
	// In other words, for this simple problem the inboard  frame and the book frame are identical.
	// Although the inboard frame can be constructed in a simple manner analogous to the outboardFrameTransformFromGround (above)
	// it is worthwhile to look at the details of the rotation matrix and position vector in the transform:
	// a. The rotation matrix relating the InboardFrame's x,y,z axes to the BookFrame's x,y,z axes is specified InboardFrame_BookFrame.
	// b. The position of the InboardFrame's origin from the BookFrame origin, expressed in terms of the BookFrame's "x, y, z" unit vectors.
	const Rotation   inboardFrameOrientationInBook;   // ( 1,0,0, 0,1,0, 0,0,1 );
	const Vec3       inboardFrameOriginLocationFromBookOrigin( 0, 0, 0 );
	const Transform  inboardFrameTransformFromBook( inboardFrameOrientationInBook, inboardFrameOriginLocationFromBookOrigin );

	// The following allows the book to move in "x", "y", and "z" directions, as well as rotate about the x,y, and z axes.
	Mobilizer bookToGroundMobilizer = Mobilizer::Planar();
	const BodyId bookBodyId = book.addRigidBody( bookMassProperties, inboardFrameTransformFromBook, GroundId, outboardFrameTransformFromGround, bookToGroundMobilizer );
	
	// Add the matter (book) sub-system to the system.
	mbs.setMatterSubsystem( book );

	// Add an air-resistance force sub-system to this multi-body system.
	GeneralForceElements userForceElements;
	mbs.addForceSubsystem( userForceElements );

	// Although "new" was used to allocate this UserForce, do not "delete" it.
	// This bug will be fixed in the next version of Simbody so it can take an object from the stack or heap.
	// For now, addUserForce takes ownership of the allocated item and takes care of deleting it at the end.
	// UserForceCostHorizontal *sinusoidalForceOnApple = new UserForceCostHorizontal( appleBodyId );
	// userForceElements.addUserForce( sinusoidalForceOnApple );

	Real springConstant = 10000;
	Real dampingRatio = 400;
	Real groundHeight = -10;
	Vec3 rHeel(-1,0,0);
	Vec3 rToe(1,0,0);
	UserForceFootVertical *footForce = new UserForceFootVertical(bookBodyId, GroundId, springConstant, dampingRatio, rHeel, rToe, groundHeight); 
	userForceElements.addUserForce( footForce );

	// Create a state for this system.
	// Define appropriate states for this multi-body system.
	// Set the initial time to 0.0
	State s;
	mbs.realize( s );
	s.setTime( 0.0 );

	// Set the initial values for the configuration variables (Euler Parameters,x,y,z)
	// The initial one is 1 due to the cosine
	book.setMobilizerQ( s, bookBodyId, 0,  ConvertFromDegreesToRadians(45) );
	book.setMobilizerQ( s, bookBodyId, 1,  0.0 );
	book.setMobilizerQ( s, bookBodyId, 2,  0.0 );
	//book.setMobilizerQ( s, bookBodyId, 3,  ConvertFromDegreesToRadians(-90.0)  );
	//book.setMobilizerQ( s, bookBodyId, 4,  0.0 );
	//book.setMobilizerQ( s, bookBodyId, 5,  0.0 );
	//book.setMobilizerQ( s, bookBodyId, 6,  0.0 );


	// Set the initial values for the motion variables (angular velocity in x/y/z, linear velocity in x/y/z)
	book.setMobilizerU( s, bookBodyId, 0,  0.0 );
	book.setMobilizerU( s, bookBodyId, 1,  0.0 );
	book.setMobilizerU( s, bookBodyId, 2,  0.0 );
//	book.setMobilizerU( s, bookBodyId, 3,  0.0 );
//	book.setMobilizerU( s, bookBodyId, 4,  0.0 );
//	book.setMobilizerU( s, bookBodyId, 5,  0.0 );

	// Create a study using the Runge Kutta Merson integrator (alternately use the CPodesIntegrator)
	RungeKuttaMerson myStudy( mbs, s );

	// Set the numerical accuracy for the integrator
	myStudy.setAccuracy( 1.0E-7 );

	// The next statement does lots of accounting
	myStudy.initialize();

	// Open a file to record the simulation results (they are also displayed on screen)
	FILE *outputFile = FileOpenWithMessageIfCannotOpen( "SpinningBookResults.txt", "w" );
	WriteStringToFile(   "time   theta    x    y  \n",  outputFile );
	WriteStringToScreen( "time   x    y    theta  \n" );

	// Set the numerical integration step and the time for the simulation to run
	const Real integrationStepDt = 0.01;
	const Real finalTime = 4.0;
	const Real finalTimeCompare = finalTime - 0.01*integrationStepDt;

	VTKReporter animationResults( mbs );

	Vec3 brickDimensions(1,3,2);
	DecorativeBrick bookBrick = DecorativeBrick(brickDimensions);
	bookBrick.setColor(Red);     // Can also specify a Vec3 with rgb which scale from 0 to 1
	bookBrick.setOpacity(0.4);   // 0.0 is solid and 1.0 is transparent
	const Rotation  identityOrientation; //( 1,0,0, 0,1,0, 0,0,1 );
	const Rotation  zRotationBy90Degrees = identityOrientation.aboutZ(ConvertFromDegreesToRadians(-90));
	const Vec3      nullLocation( 0, 0, 0 );

	const Transform identityTransform( identityOrientation, nullLocation );
	const Transform rotationBy90DegreesZTransform( zRotationBy90Degrees, nullLocation );

	animationResults.addDecoration( bookBodyId, rotationBy90DegreesZTransform, bookBrick );

	Vec3 groundDimensions(10,.2,10);
	DecorativeBrick groundBrick = DecorativeBrick(groundDimensions);
	groundBrick.setColor(Blue);
	groundBrick.setOpacity(0.5);
	const Vec3 groundLocation(0,-10,0);
	const Transform groundTransform(identityOrientation, groundLocation);
	animationResults.addDecoration(GroundId, groundTransform, groundBrick);


	// Run the simulation and print the results
	while( 1 )
	{
		// Query for results to be printed
		Real time = s.getTime();
		Real kineticEnergy = mbs.getKineticEnergy(s); 

		// Get the book angular velocity in ground, expressed in terms of the ground's "x,y,z" unit vectors.
		// Convert to book angular velocity in book, expressed in terms of the book's x/y/z unit vectors
		// Extract the book's x,y, and z angular velocities from this vector.
		const Vec3 bookAngularVelocity_Ground = book.calcBodyAngularVelocityInBody( s, bookBodyId, GroundId );
		const Rotation R_Ground_Book = book.getBodyRotation(s, bookBodyId);
		const Vec3 bookAngularVelocity_Book = R_Ground_Book.invert()*bookAngularVelocity_Ground;
		Real wx = bookAngularVelocity_Book[0];
		Real wy = bookAngularVelocity_Book[1];
		Real wz = bookAngularVelocity_Book[2];

		const Real bookX = book.getMobilizerQ(s, bookBodyId, 0);
		const Real bookY = book.getMobilizerQ(s, bookBodyId, 1);
		const Real bookTheta = book.getMobilizerQ(s, bookBodyId, 2);


		// Calculate the angular momentum from the inertia matrix and the angular velocities.
		Real Hx = Ixx*wx;
		Real Hy = Iyy*wy;
		Real Hz = Izz*wz;
		Real totalH = sqrt(pow(Hx,2)+pow(Hy,2)+pow(Hz,2));


		// Print results to screen
/*		WriteDoubleToFile( time,			    2, stdout );
		WriteDoubleToFile( bookX,				4, stdout );
		WriteDoubleToFile( bookY,				4, stdout );
		WriteDoubleToFile( bookTheta,			4, stdout );
		WriteStringToScreen( "\n" );
*/		// Print results to file
		WriteDoubleToFile( time,			    2, outputFile );
		WriteDoubleToFile( bookX,				4, outputFile );
		WriteDoubleToFile( bookY,				4, outputFile );
		WriteDoubleToFile( bookTheta,			4, outputFile );
		WriteStringToFile( "\n", outputFile );

		animationResults.report(s);

		// Check if integration has completed
		if( time >= finalTimeCompare ) break;

		// Increment time step
		myStudy.step( time + integrationStepDt );
	}

	// Simulation completed properly
	return true;
}


//-----------------------------------------------------------------------------
FILE*  FileOpenWithMessageIfCannotOpen( const char *filename, const char *attribute )
{
	// Try to open the file
	FILE *Fptr1 = fopen( filename, attribute );

	// If unable to open the file, issue a message
	if( !Fptr1 )
	{
		WriteStringToScreen( "\n\n Unable to open the file: " );
		WriteStringToScreen( filename );
		WriteStringToScreen( "\n\n" );
	}

	return Fptr1;
}


//-----------------------------------------------------------------------------
bool  WriteDoubleToFile( double x, int precision, FILE *fptr )
{
	// Ensure the precision (number of digits in the mantissa after the decimal point) makes sense.
	// Next, calculate the field width so it includes one extra space to the right of the number.
	if( precision < 0 || precision > 17 ) precision = 5;
	int fieldWidth = precision + 8;

	// Create the format specifier and print the number
	char format[20];
	sprintf( format, " %%- %d.%dE", fieldWidth, precision );
	return fprintf( fptr, format, x ) >= 0;
}


